<?php
/***************************************************************************************************
Name:              DynDNS.php
Revision:          1.01
Author:            R.J. Vrijhof
Date:              Nov 15, 2000
Uses:              PHP 4
PHP Include files: http.php
Email address:     R.J.Vrijhof@bigfoot.com
Web site:          http://www.rjvrijhof.com/ or http://richy.webhop.net/
Download from:     http://www.rjvrijhof.com/download/
****************************************************************************************************
                                              Description

This script is a simple DynDNS client written in PHP 4. You can use it if you have a xxx.dyndns.org
subdomain to automatically update your IP address in the DynDNS NIC when it changes.
It was originally written for PHP 4 running on NT 4 Server, but it should also run on any UNIX
flavour and with minor modifications, it should also run with PHP 3 (PHP 3 doesn't know the syntax
"foreach" used in this script, so you have to modify that into something which uses a combination of
"list()" and "each()").
Basically, it sleeps for a certain amount of time (normally 10 seconds) and checks whether the IP
address has changed. If it has, it updates the IP address to the DynDNS NIC and writes the reply to
the screen. Then it sleeps again, normally for 10 seconds, but if the DynDNS NIC says it has to wait
a different amount of time, it does just that (as is required by DynDNS). If there is an
unrecoverable error reply from the DynDNS NIC, it exits, because that error has to be solved first,
or the DynDNS status page is telling everything is alright again.
There's no possibily to do a forced update. To accomplish that, you have to stop the script,
delete the IP file and start the script again. Watch out with this one though, or your DynDNS
account will be blocked for abuse! Or you could reconnect to the internet, then normally your IP
address is changed, so the script should update it...

To use it, do the following:
-Make a directory to hold the PHP and IP files (which this script produces)
-Copy the PHP files (DynDNS.php and config.inc.php) into the directory you just made
-Copy the HTTP Class file (http.php) to your include directory or in the same directory as the other
 files
-Change the variables in config.inc.php to the right values
-Run it (preferably, you could run it automatically when booting up)

You should run this script from the command prompt, not from a browser.

This script was made specifically for DynDNS NIC 2!

****************************************************************************************************
                                          Copyright/License

Copyright (C) 2000  R.J. Vrijhof
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
You can also find the GNU GPL license on the Web: http://www.gnu.org/copyleft/gpl.html.

****************************************************************************************************
                                          Revision history

1.0     Oct 29, 2000    Initial version
1.01    Nov 15, 2000    Fixed stupid bug (didn't do any harm, but it was still stupid)
****************************************************************************************************
***************************************************************************************************/

require("config.inc.php");

/***************************************************************************************************
Beyond this point nothing should be changed normally.
***************************************************************************************************/
// This script makes use of the HTTP class from Manuel Lemos (thanks a lot, Manuel! Great class!)
require("http.php");
$http = new http_class;
// Because this script should be running endlessly, disable PHP timeout
set_time_limit(0);
// Start of the endless loop
for (;;) {
  $delay = $initDelay;
  // Get the IP address(es)
  $IParr = gethostbynamel($subdomain);
  // Get the internet IP address (if you have more than one IP address)
  foreach ($IParr as $value)
    if (ereg($IPregex, $value))
      $IP = $value;
  // If the IP file with this IP address doesn't exist, then the IP address has changed, so update it
  if (!file_exists("$path\\$IP")) {
    // Make the new IP file
    $fp = fopen("$path\\$IP", "w");
    fclose($fp);
    // Delete the old IP file (if existent)
    $dp = opendir($path);
    while ($dir = readdir($dp))
      if (ereg($IPregex, $dir) && $dir != $IP)
        unlink("$path\\$dir");
    closedir($dp);
    // Establish an HTTP connection to the DynDNS member site
    $http->host_port = $port;
    $error = $http->Open(array("HostName" => $host));
    if (!$error) {
      // Use HTTP 1.1 protocol
      $http->protocol_version = "1.1";
      // Send the updated IP address to the DynDNS NIC
      $error = $http->SendRequest(array(
        "RequestURI" => $URI,
        "RequestMethod" => "POST",
        "PostValues" => array(
          "system" => "dyndns",
          "hostname" => $subdomain,
          "myip" => $IP,
          "wildcard" => "OFF",
          "mx" => "",
          "backmx" => "NO",
        ),
        "Headers" => array(
          "Authorization" => "Basic ".base64_encode("$login:$pass"),
          "User-Agent" => $ua,
          "Pragma" => "no-cache"
        )
      ));
      if (!$error) {
        // Get the reply headers
        $error = $http->ReadReplyHeaders(&$headers);
        if (!$error && eregi("^http/1\.1 *200 *ok", key($headers))) {
          $body = "";
          // Get the reply body
          for (;;) {
            $error = $http->ReadReplyBody(&$buffer, 1000);
            if ($error || !$buffer)
              break;
            $body .= $buffer;
          }
          if (!$error) {
            $bodyArr = split("\r\n", $body);
            $result = chop($bodyArr[1]);
            $resArr = split(" ", $result);
            $resArr[0] = strtolower($resArr[0]);
            // Test for the reply from the DynDNS NIC and write the result to the (screen) log
            switch ($resArr[0]) {
              case "good" :
                echo date("M d, Y")." at ".date("H:i:s").": DynDNS IP address updated to ".$resArr[1]."\n";
                break;
              case "badauth" :
                die(date("M d, Y")." at ".date("H:i:s").": Invalid username or password\n");
              case "badsys" :
                die(date("M d, Y")." at ".date("H:i:s").": Invalid system parameter\n");
              case "nochg" :
                echo date("M d, Y")." at ".date("H:i:s").": DynDNS was already updated to ".$resArr[1]."\n";
                break;
              case "notfqdn" :
                die(date("M d, Y")." at ".date("H:i:s").": Subdomain not fully qualified\n");
              case "nohost" :
                die(date("M d, Y")." at ".date("H:i:s").": Subdomain does not exist\n");
              case "!yours" :
                die(date("M d, Y")." at ".date("H:i:s").": Wrong username for subdomain\n");
              case "abuse" :
                die(date("M d, Y")." at ".date("H:i:s").": Subdomain blocked for abuse\n                          Contact support@dyndns.org to unblock\n");
              case "numhost" :
                die(date("M d, Y")." at ".date("H:i:s").": Too many or too few subdomains\n                          Contact support@dyndns.org\n");
              case "dnserr" :
                die(date("M d, Y")." at ".date("H:i:s").": DNS error \"".$resArr[1]."\" encountered\n                          Contact support@dyndns.org\n");
              case "911" :
                die(date("M d, Y")." at ".date("H:i:s").": Shutdown till further notice\n");
              case "999" :
                die(date("M d, Y")." at ".date("H:i:s").": Shutdown till further notice\n");
              default :
                if (ereg("w([0-9]+)h", $resArr[0], $regs)) {
                  $delay = $regs[1] * 60 * 60;
                  echo date("M d, Y")." at ".date("H:i:s").": Delay for ".$regs[1]." hours\n";
                  if ($resArr[1])
                    echo "                          Reason: ".$resArr[1]."\n";
                } elseif (ereg("w([0-9]+)m", $resArr[0], $regs)) {
                  $delay = $regs[1] * 60;
                  echo date("M d, Y")." at ".date("H:i:s").": Delay for ".$regs[1]." minutes\n";
                  if ($resArr[1])
                    echo "                          Reason: ".$resArr[1]."\n";
                } elseif (ereg("w([0-9]+)s", $resArr[0], $regs)) {
                  $delay = $regs[1];
                  echo date("M d, Y")." at ".date("H:i:s").": Delay for ".$regs[1]." seconds\n";
                  if ($resArr[1])
                    echo "                          Reason: ".$resArr[1]."\n";
                } elseif (ereg("wu([0-9][0-9])([0-9][0-9])", $resArr[0], $regs)) {
                  if (date("G") > ($regs[1] + $tz) || (date("G") == ($regs[1] + $tz) && date("i") > $regs[2])) {
                    $delay = mktime($regs[1] + $tz, $regs[2], 0, date("m"), date("j") + 1, date("Y")) - time();
                    echo date("M d, Y")." at ".date("H:i:s").": Delay until ".date("H", time() + $delay).":".date("i", time() + $delay)." tomorrow\n";
                    if ($resArr[1])
                      echo "                          Reason: ".$resArr[1]."\n";
                  } else {
                    $delay = mktime($regs[1] + $tz, $regs[2], 0, date("m"), date("j"), date("Y")) - time();
                    echo date("M d, Y")." at ".date("H:i:s").": Delay until ".date("H", time() + $delay).":".date("i", time() + $delay)."\n";
                    if ($resArr[1])
                      echo "                          Reason: ".$resArr[1]."\n";
                  }
                } else
                  echo date("M d, Y")." at ".date("H:i:s").": Unrecognized reply from DynDNS NIC\n";
            }
          } else
            echo date("M d, Y")." at ".date("H:i:s").": HTTP error getting the reply body\n";
        } else
          echo date("M d, Y")." at ".date("H:i:s").": HTTP error getting the reply headers\n";
      } else
        echo date("M d, Y")." at ".date("H:i:s").": HTTP error sending the update to the DynDNS NIC\n";
    } else
      echo date("M d, Y")." at ".date("H:i:s").": HTTP error connecting to the DynDNS member site\n";
    // Close the HTTP connection
    $http->Close();
  }
  // Wait until it's time to check the IP address again
  sleep($delay);
}
?>
